package net.knarcraft.blacksmith.command;

import net.knarcraft.blacksmith.BlacksmithPlugin;
import net.knarcraft.blacksmith.config.GlobalSetting;
import net.knarcraft.blacksmith.config.GlobalSettings;
import net.knarcraft.blacksmith.config.NPCSetting;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.enchantments.Enchantment;
import org.jetbrains.annotations.NotNull;

/**
 * The command used for changing global configuration options
 */
public class BlackSmithConfigCommand implements CommandExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                             @NotNull String[] args) {
        if (args.length == 0) {
            return false;
        }

        String commandName = args[0];
        if (commandName.equalsIgnoreCase("reload")) {
            return new ReloadCommand().onCommand(sender, command, label, args);
        }
        GlobalSettings settings = BlacksmithPlugin.getInstance().getSettings();

        //Changing reforge-able items' default isn't recommended
        if (commandName.equalsIgnoreCase(NPCSetting.REFORGE_ABLE_ITEMS.getCommandName())) {
            sender.sendMessage(ChatColor.DARK_RED + "Changing reforge-able items globally will make every new " +
                    "blacksmith unable to re-forge anything not in the list, unless it's changed for the " +
                    "individual NPC. If you really want to change this, change it manually.");
            return false;
        }

        //Find which global setting the user has specified, if any
        GlobalSetting detectedGlobalSetting = null;
        for (GlobalSetting globalSetting : GlobalSetting.values()) {
            if (commandName.equalsIgnoreCase(globalSetting.getCommandName())) {
                detectedGlobalSetting = globalSetting;
                break;
            }
        }

        //Find which npc setting the user has specified, if any
        NPCSetting detectedNPCSetting = null;
        for (NPCSetting npcSetting : NPCSetting.values()) {
            if (commandName.equalsIgnoreCase(npcSetting.getCommandName())) {
                detectedNPCSetting = npcSetting;
                break;
            }
        }

        if (args.length == 1) {
            if (detectedGlobalSetting != null) {
                sender.sendMessage(String.format("Current value of %s: %s", detectedGlobalSetting.getCommandName(),
                        settings.getRawValue(detectedGlobalSetting)));
            } else if (detectedNPCSetting != null) {
                sender.sendMessage(String.format("Current value of %s: %s", detectedNPCSetting.getCommandName(),
                        settings.getRawValue(detectedNPCSetting)));
            }
            return true;
        } else if (args.length == 2 && isSpecialCase(commandName)) {
            return displaySpecialCaseValue(args[1], sender, detectedGlobalSetting, settings);
        }

        if (updateSpecialCase(settings, commandName, args)) {
            return true;
        }

        if (detectedGlobalSetting != null) {
            settings.changeValue(detectedGlobalSetting, args[1]);
            return true;
        } else if (detectedNPCSetting != null) {
            settings.changeValue(detectedNPCSetting, args[1]);
            return true;
        }
        return false;
    }

    /**
     * Displays the current value of a special-case configuration value
     *
     * @param selector <p>The selector specifying which material/enchantment to display information about</p>
     * @param sender   <p>The sender to display the value to</p>
     * @param setting  <p>The setting to display</p>
     * @param settings <p>The settings object to query</p>
     * @return <p>True if the value was successfully displayed</p>
     */
    private boolean displaySpecialCaseValue(String selector, CommandSender sender, GlobalSetting setting,
                                            GlobalSettings settings) {
        if (setting == GlobalSetting.BASE_PRICE) {
            Material material = Material.matchMaterial(selector);
            if (material == null) {
                return false;
            }
            sender.sendMessage(String.format("Current value of %s for material %s: %s", setting.getCommandName(),
                    material, settings.getBasePrice(material)));
            return true;
        } else if (setting == GlobalSetting.PRICE_PER_DURABILITY_POINT) {
            Material material = Material.matchMaterial(selector);
            if (material == null) {
                return false;
            }
            sender.sendMessage(String.format("Current value of %s for material %s: %s", setting.getCommandName(),
                    material, settings.getPricePerDurabilityPoint(material)));
        } else if (setting == GlobalSetting.ENCHANTMENT_COST) {
            Enchantment enchantment = Enchantment.getByKey(NamespacedKey.minecraft(selector));
            if (enchantment == null) {
                return false;
            }
            sender.sendMessage(String.format("Current value of %s for enchantment %s: %s", setting.getCommandName(),
                    enchantment, settings.getEnchantmentCost(enchantment)));
        }
        return false;
    }

    /**
     * Gets whether a command name matches a special case command
     *
     * @param commandName <p>The command specified</p>
     * @return <p>True if the command is a special case</p>
     */
    private boolean isSpecialCase(String commandName) {
        return commandName.equalsIgnoreCase(GlobalSetting.BASE_PRICE.getCommandName()) ||
                commandName.equalsIgnoreCase(GlobalSetting.PRICE_PER_DURABILITY_POINT.getCommandName()) ||
                commandName.equalsIgnoreCase(GlobalSetting.ENCHANTMENT_COST.getCommandName());
    }

    /**
     * Updates a special-case configuration value if a special-case is encountered
     *
     * @param settings    <p>The settings to modify</p>
     * @param commandName <p>The sub-command the player specified</p>
     * @param args        <p>All arguments given</p>
     * @return <p>True if already handled as a special case</p>
     */
    private boolean updateSpecialCase(GlobalSettings settings, String commandName, String[] args) {
        if (commandName.equalsIgnoreCase(GlobalSetting.BASE_PRICE.getCommandName())) {
            Material material = Material.matchMaterial(args[1]);
            if (material == null) {
                return false;
            }
            settings.setBasePrice(material, Double.parseDouble(args[2]));
            return true;
        }

        if (commandName.equalsIgnoreCase(GlobalSetting.PRICE_PER_DURABILITY_POINT.getCommandName())) {
            Material material = Material.matchMaterial(args[1]);
            if (material == null) {
                return false;
            }
            settings.setPricePerDurabilityPoint(material, Double.parseDouble(args[2]));
            return true;
        }

        if (commandName.equalsIgnoreCase(GlobalSetting.ENCHANTMENT_COST.getCommandName())) {
            Enchantment enchantment = Enchantment.getByKey(NamespacedKey.minecraft(args[1]));
            if (enchantment == null) {
                return false;
            }
            settings.setEnchantmentCost(enchantment, Double.parseDouble(args[2]));
            return true;
        }

        return false;
    }

}
